/*
	NOTICE:

	THIS INFORMATION IS CONFIDENTIAL AND PROPRIETARY TO
	COMPUTER GENERATION, INC. AND MAY NOT BE COPIED,
	MODIFIED, OR OTHERWISE USED.
*/

/*
********************************************************************************
;+
LOCK_CHECK	-	Check for advisory locks on one or more files

FILE:		$RCSfile: lock_check.c,v $

DESCRIPTION:	Check a filename for any fcntl() style locks pending.

AUTHOR:		Eric Ayers

FUNCTIONS:	n/a

USAGE:		lock_check [-v] <file list>

		where:	-v		Verbose mode.  If set prints system ID,
					lock offset, and length.

			<file_list>	List of one or more files to check on.
;-
********************************************************************************
*/



/*
********************************************************************************

Last Revision:

$log
	Add -l and -s options
$log

********************************************************************************

Revision History:

$Log: lock_check.c,v $
Revision 1.8  2007/06/15 15:15:31  eric
Added errno, strerror() values when fcntl() fails for F_GETLK.

Revision 1.7  2006/10/19 21:05:53  eric
fixed a bunch of compiler warnings.

Revision 1.6  1999/02/17 22:00:29  gary
	Add -l and -s options

 Revision 1.5  1998/07/29  00:19:00  eric
 Added a debugging message.

Revision 1.4  1996/10/09 15:10:07  greg
	Add header, verbose mode, and output of lock offset and length.


********************************************************************************
*/

#define VERSION		"$Revision: 1.8 $"
#ident	"@(#)$Id: lock_check.c,v 1.8 2007/06/15 15:15:31 eric Exp $ 	 $Revision: 1.8 $"



/*** FILES INCLUDED ***********************************************************/

/* System definitions							*/

#ifdef __cplusplus
extern "C" {	/* Wrap function prototypes for use with C++ 		*/
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
	
#ifdef __cplusplus
}	/* Close scope of 'extern "C"' declaration 			*/
#endif



static void usage (char * progname)
{
	fprintf (stderr, "\
usage:    %s [-v] [-s nnn] [-l nnn]  <file list>\n\
where:    -v Verbose mode.  If set displays start and length of locked\n\
               region as well as listing files for which no locks are held.\n\
          -s nn start of lock to test (default 0)\n\
	  -l nn length of lock to test (default entire file\n\
          <file list> Lists files to check on.\n\
Tests the specified filenames for fcntl () advisory locks\n\
",progname);
}



int main (int argc, char * argv[])
{
	int i, file_cnt = 0, lock_cnt = 0, verbose = 0;
	extern int errno;
	int l_start=0, l_len=0;
	if (argc == 1)
	{
		usage (argv[0]);
		return 1;
	}

	/* Loop through arguments - either -v flag or file name 	*/

	for (i = 1; i < argc; i++)
	{
#ifdef OSF
		struct eflock flock_buf;
#else
		struct flock flock_buf;
#endif
		int fd, return_val;
		
		/* Test for verbose mode flag				*/
		if (strcmp ("-v", argv[i]) == 0)
		{
			verbose = 1;
			continue;
		}

		if (strcmp ("-l", argv[i]) == 0)
		{
			i++;
			l_len = atoi(argv[i]);
			continue;
		}

		if (strcmp ("-s", argv[i]) == 0)
		{
			i++;
			l_start = atoi(argv[i]);
			continue;
		}

		/* Must be a file name - open the file			*/
		if (verbose)
		{
			fprintf (stderr, "Trying: %s start %d length %d\n", argv[i], l_start, l_len);
			fflush (stderr);
		}
		
		if ((fd = open (argv[i], O_RDWR)) < 0)
		{
			int	old_errno = errno;
			char	* errval = strerror (errno);
			
			fprintf (stderr,
				 "open of file %s failed with error %d : %s\n",
				 argv[i],
				 old_errno,
				 errval);
			
			continue;

		}

		file_cnt++;
		
		/* Test for impediments to writelocking entire file	*/

		memset (&flock_buf, 0, sizeof (flock_buf));

		flock_buf.l_type   = F_WRLCK;
		flock_buf.l_whence = SEEK_SET;
		flock_buf.l_start  = l_start;
		flock_buf.l_len    = l_len;
		
		return_val = fcntl (fd, F_GETLK, &flock_buf);

		if (return_val)
		{
			fprintf(stderr,
				"fcntl failed for F_GETLK. Errno %d (%s)\n",
				errno, strerror(errno));
		}
		else
		{
			if ((flock_buf.l_type == F_UNLCK) && verbose)
			{
				printf ("No process locking file \"%s\"\n",
					argv[i]);
			}
			else if (flock_buf.l_type != F_UNLCK)
			{
				char *lock_type_desc;

				lock_cnt++;

				if (flock_buf.l_type & F_WRLCK)
				{
					lock_type_desc = "WRITE";
				}
				else if (flock_buf.l_type & F_RDLCK)
				{
					lock_type_desc = "READ";
				}
				else
				{
					lock_type_desc = "UNKNOWN";
				}
#ifdef OSF
				printf ("File: \"%s\" %s locked by:\npid: %d ",
					argv[i], lock_type_desc,
					flock_buf.l_pid);
					
				if ((flock_buf.l_rpid != 0) ||
				    (flock_buf.l_rsys != 0))
				{
					printf ("REMOTELY - remote pid: %d, "
						"sysid: %08X\n",
					flock_buf.l_rpid,
					(flock_buf.l_rsys));
				}
				else
				{
					printf ("\n");
				}

#else
#ifdef UNIX_SV
				printf ("File: \"%s\" %s locked by:\npid: %d ",
					argv[i],
					lock_type_desc,
					flock_buf.l_pid);
					
				if (flock_buf.l_sysid != 0)
				{
					printf ("REMOTELY - sysid: %08X\n",
					(flock_buf.l_sysid));
				}
				else
				{
					printf ("\n");
				}
#else
				printf ("File: \"%s\" locked by:\npid: %d\n"
					"type: %s\n",
					argv[i],
					flock_buf.l_pid,
					lock_type_desc);
#endif
#endif
				if (verbose)
				{
					printf ("Start: 0x%lX Length: 0x%lX\n",
						(long)flock_buf.l_start,
						(long)flock_buf.l_len);
				}				

				/* What in the hell is that l_sysid???
				 * it doesn't look like an IP address...
				 */
/*				printf ("File: \"%s\" locked by:\n\
sysid: %03u.%03u.%03u.%03u\npid: %d\ntype: %s\n",
					argv[i],
					(flock_buf.l_sysid & 0xff000000) >> 24,
					(flock_buf.l_sysid & 0x00ff0000) >>16,
					(flock_buf.l_sysid & 0x0000ff00) >> 8,
					(flock_buf.l_sysid & 0x000000ff),
					flock_buf.l_pid,
					lock_type_desc);
*/
			}
		}
		
		close (fd);
	}

	printf ("\n%d of %d files opened had locks set\n\n", lock_cnt,
		file_cnt);

	return 0;
	
} /* end main() */
